/* log.c */

/*
  SquidClamav - A Clamav Antivirus Redirector for Squid

  Copyright (C) 2005-2010 Gilles Darold

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see < http://www.gnu.org/licenses/ >.

*/


#include"squidclamav.h"
#include"log.h"
#include"util.h"
#include<stdio.h>
#include<time.h>
#include<stdlib.h>
#include<string.h>
#include<sys/types.h>
#include<unistd.h>

#define MAXPATHLEN 256

static char *get_date(void);


extern int interactive;       /* from squidclamav.c */
extern int debug;             /* from squidclamav.c */
extern char *log_file;        /* from squidclamav.c */



void
logit(char *filename, char *format, ...)
{
  FILE *log;
  char *date_str = NULL;
  char *msg = NULL;
  va_list args;

  va_start(args, format);
  msg = (char *)malloc(sizeof(char)*MAX_URL);
  if (vsnprintf(msg, MAX_URL, format, args) < 0) {
    /* Can't format log, so just return */
    xfree(msg);
    return;
  }
  va_end(args);

  date_str = get_date();
  chomp(date_str);
  if ( (interactive != 0) || (filename == NULL) ) {
    fprintf(stderr, "%s %s", date_str, msg);
    if (fflush(stderr) != 0)
       fprintf(stderr, "%s ERROR %s", date_str, "Error flushing log stream\n");
  } else {
    /* ensure that this is the good file (i-e not symlink nor device) */
    if (isPathSecure(filename) == 0) {
	    log = fopen(filename, "at");
	    if (log == NULL) {
		xfree(log_file);
		log_file = NULL;
		fprintf(stderr, "%s ERROR %s", date_str, "Error opening log stream\n");
		fprintf(stderr, "%s %s", date_str, msg);
	    } else {
		    
		    fprintf(log, "%s [%d] %s", date_str, (int)getpid(), msg);
		    if (fflush(log) != 0)
			fprintf(stderr, "%s ERROR %s", date_str, "Error flushing log stream\n");
		    if (fclose(log) != 0)
			fprintf(stderr, "%s ERROR %s", date_str, "Error closing log stream\n");
		}
    } else {
	    fprintf(stderr, "%s ERROR %s", date_str, "Error log file is unsecure\n");
	}
  }
  xfree(msg);
}


char *
get_date(void)
{
  time_t tp;
  tp = time(NULL);

  return ctime(&tp);
}

