/* pattern.c */
   
/*
  SquidClamav - A Clamav Antivirus Redirector for Squid

  Copyright (C) 2005-2010 Gilles Darold

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see < http://www.gnu.org/licenses/ >.

*/


#include"squidclamav.h"
#include"log.h"
#include"util.h"

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#include<ctype.h>

extern int debug;               /* from squidclamav.c  */
extern char *log_file;          /* from squidclamav.c  */
extern SCPattern *patterns;     /* from squidclamav.c */
extern int pattc;               /* from squidclamav.c */
extern struct IN_BUFF in_buff;  /* from squidclamav.c */

int
isIpAddress(char *src_addr)
{
  char *ptr;
  int address;
  int i;
  char *s = (char *) malloc (sizeof (char) * LOW_CHAR);

  xstrncpy(s, src_addr, LOW_CHAR);
  
  /* make sure we have numbers and dots only! */
  if(strspn(s, "0123456789.") != strlen(s)) {
    xfree(s);
    xfree(ptr);
    return 1;
  }

  /* split up each number from string */
  ptr = strtok(s, ".");
  if(ptr == NULL) {
    xfree(s);
    return 1;
  }
  address = atoi(ptr);
  if(address < 0 || address > 255) {
    xfree(s);
    xfree(ptr);
    return 1;
  }
  
  for(i = 2; i < 4; i++) {
    ptr = strtok(NULL, ".");
    if (ptr == NULL) {
       xfree(s);
       return 1;
    }
    address = atoi(ptr);
    if (address < 0 || address > 255) {
       xfree(ptr);
       xfree(s);
       return 1;
    }
  }
  xfree(s);
  
  return 0;
}

int
parseSquidEntry( char* str, const char* delim)
{
	if ( (str == NULL) || (strcmp(str, "") == 0)) {
	   if (debug != 0)
		logit(log_file, "DEBUG parseSquidEntry: empty squid input.\n");
	   return 1;
	}

	char *s = (char *) malloc (sizeof (char) * (MAX_URL*2));
	xstrncpy(s, str, MAX_URL*2);

	xstrncpy(in_buff.url, strtok(s, delim), MAX_URL);
	xstrncpy(in_buff.src_address, strtok(NULL, delim), SMALL_BUFF+16);
	xstrncpy(in_buff.ident, strtok(NULL, delim), MAX_LOGIN_SZ);
	xstrncpy(in_buff.method, strtok(NULL, delim), LOW_CHAR);
	xfree(s);

	return 0;
}


int
parseSourceAddress( char* str, const char* delim)
{
	if ( (str == NULL) || (strcmp(str, "") == 0)) {
	   if (debug != 0)
		logit(log_file, "DEBUG parseSourceAddress: empty squid input.\n");
	   return 1;
	}

	char *s = (char *) malloc (sizeof (char) * (SMALL_BUFF+16));
	xstrncpy(s, str, SMALL_BUFF+16);

	xstrncpy(in_buff.ipaddress, strtok(s, delim), 16);
	xstrncpy(in_buff.fqdn, strtok(NULL, delim), SMALL_BUFF);
	xfree(s);

	return 0;
}


/* load the stdin for the redirector into an IN_BUFF structure.
   Return 1 if the fields can't be converted
   Return -1 if method is not GET
   Return 0 if all fields are Ok
*/
int
load_in_buff(char *buff)
{
  if ( (buff == NULL) || (strcmp(buff, "") == 0)) {
     if (debug != 0)
	logit(log_file, "DEBUG load_in_buff: empty squid input.\n");
     return 1;
  }

  /* parse squid entry into in_buff */
  if (parseSquidEntry(buff, " ") != 0) {
     return 1;
  }
  
  /* extract source ipaddress and source fqdn */
  if (parseSourceAddress(in_buff.src_address, "/") != 0) {
     return 1;
  }
  
  /* extract source ipaddress and source fqdn */
  if (parseSourceAddress(in_buff.src_address, "/") != 0) {
  }
  
  if (debug != 0)
	logit(log_file, "DEBUG Parsed request: %s %s/%s %s %s\n", in_buff.url, in_buff.ipaddress, in_buff.fqdn, in_buff.ident, in_buff.method);


  /* Try to find error condition in parsed squid input */
  if (xstrnlen(in_buff.url, MAX_URL) <= 4) {
     logit(log_file, "ERROR strlen in_buff.url = [%d] in main()\n", 
	xstrnlen(in_buff.url, MAX_URL));
     return 1;
  }

  /* check that the IP source address supplied is valid */
  if (isIpAddress(in_buff.ipaddress) != 0) {
     logit(log_file, "ERROR in_buff.src_address not valid: %s\n", in_buff.src_address);
     return 1;
  } 

  /* all methods must be GET */
  if (strcmp(in_buff.method, "GET") != 0) {
      if (debug != 0)
	 logit(log_file, "DEBUG in_buff.method not \"GET\" (%s)\n", in_buff.method);
      return -1;
  }
  
  /* Try to find error condition in parsed squid input */
  if (xstrnlen(in_buff.url, MAX_URL) <= 4) {
     logit(log_file, "ERROR strlen in_buff.url = [%d] in main()\n", 
	xstrnlen(in_buff.url, MAX_URL));
     return 1;
  }

  /* check that the IP source address supplied is valid */
  if (isIpAddress(in_buff.ipaddress) != 0) {
     logit(log_file, "ERROR in_buff.src_address not valid: %s\n", in_buff.src_address);
     return 1;
  } 

  /* all methods must be GET */
  if (strcmp(in_buff.method, "GET") != 0) {
      if (debug != 0)
	 logit(log_file, "DEBUG in_buff.method not \"GET\" (%s)\n", in_buff.method);
      return -1;
  }
  
  return 0;
}

int
simple_pattern_compare(char *str, const int type)
{
	int i = 0;  

	/* pass througth all regex pattern */
	for (i = 0; i < pattc; i++) {
		if ( (patterns[i].type == type) && (regexec(&patterns[i].regexv, str, 0, 0, 0) == 0) ) {
			switch(type) {
				/* return 1 if string matches whitelist pattern */
				case WHITELIST:
					if (debug > 0)
						logit(log_file, "DEBUG whitelist (%s) matched: %s\n", patterns[i].pattern, str);
					return 1;
				/* return 1 if string matches abort pattern */
				case ABORT:
					if (debug > 0)
						logit(log_file, "DEBUG abort (%s) matched: %s\n", patterns[i].pattern, str);
					return 1;
				/* return 1 if string matches trustuser pattern */
				case TRUSTUSER:
					if (debug > 0)
						logit(log_file, "DEBUG trustuser (%s) matched: %s\n", patterns[i].pattern, str);
					return 1;
				/* return 1 if string matches abortcontent pattern */
				case ABORTCONTENT:
					if (debug > 0)
						logit(log_file, "DEBUG abortcontent (%s) matched: %s\n", patterns[i].pattern, str);
					return 1;
				default:
					logit(log_file, "ERROR unknown pattern match type: %s\n", str);
					return -1;
			}
		}
	}

	/* return 0 otherwise */
	return 0;
}

int
client_pattern_compare(char *ip, char *name)
{
	int i = 0;  

	/* pass througth all regex pattern */
	for (i = 0; i < pattc; i++) {
		if (patterns[i].type == TRUSTCLIENT) {
			/* Look at client ip pattern matching */
			/* return 1 if string matches ip TRUSTCLIENT pattern */
			if (regexec(&patterns[i].regexv, ip, 0, 0, 0) == 0) {
				if (debug != 0)
					logit(log_file, "DEBUG trustclient (%s) matched: %s\n", patterns[i].pattern, ip);
				return 1;
			/* Look at client name pattern matching */
			/* return 2 if string matches fqdn TRUSTCLIENT pattern */
			} else if (regexec(&patterns[i].regexv, name, 0, 0, 0) == 0) {
				if (debug != 0)
					logit(log_file, "DEBUG trustclient (%s) matched: %s\n", patterns[i].pattern, name);
				return 2;
			}
		}
	}

	/* return 0 otherwise */
	return 0;
}

