/* scconfig.c */

/*
  SquidClamav - A Clamav Antivirus Redirector for Squid

  Copyright (C) 2005-2010 Gilles Darold

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see < http://www.gnu.org/licenses/ >.

*/

#include<string.h>
#include<stdio.h>
#include<stdlib.h>
#include<ctype.h>
#include"squidclamav.h"
#include"log.h"
#include"util.h"


extern int bridge_mode;		/* from squidclamav.c */
extern char *alternate_config;	/* from squidclamav.c */
extern char *redirect_url;	/* from squidclamav.c */
extern char *squidguard;	/* from squidclamav.c */
extern int debug;		/* from squidclamav.c */
extern int logredir;		/* from squidclamav.c */
extern int statit;		/* from squidclamav.c */
extern char *log_file;		/* from squidclamav.c */
extern char *proxy;		/* from squidclamav.c */
extern char *clamd_ip;		/* from squidclamav.c */
extern char *clamd_port;	/* from squidclamav.c */
extern char *clamd_local;	/* from squidclamav.c */
extern int timeout;		/* from squidclamav.c */
extern char *useragent;		/* from squidclamav.c */
extern int maxredir;		/* from squidclamav.c */
extern int trust_cache;		/* from squidclamav.c */
extern char *squid_ip;		/* from squidclamav.c */
extern char *squid_port;	/* from squidclamav.c */
extern SCPattern *patterns;	/* from squidclamav.c */
extern int pattc;               /* from squidclamav.c */
extern int current_pattern_size;/* from squidclamav.c */
extern double maxsize;          /* from squidclamav.c */
const char *DefaultConfigFile = DEFAULT_CONFIG_FILE;
extern int nohead;              /* from squidclamav.c */

/* load the squidclamav.conf */
void
load_patterns()
{
  char *buf = NULL;
  FILE *fp  = NULL;

  /* from squidclamav.c - main(), use alternate config if given */
  if ( (alternate_config != NULL) && (isPathExists(alternate_config) == 0) ) { 
    fp = fopen(alternate_config, "rt");
    if (debug > 0)
       logit(log_file, "LOG Reading configuration from alternate_config: %s\n", alternate_config);
  } else if ((alternate_config == NULL) && (isPathExists(DefaultConfigFile) == 0)) {
    fp = fopen(DefaultConfigFile, "rt");
    if (debug > 0)
       logit(log_file, "LOG Reading configuration from DefaultConfigFile: %s\n", CONFIG_FILE);
  }
  

  if (fp == NULL) {
    if (alternate_config != NULL)
	logit(log_file, "PANIC unable to open configuration file: %s\n", alternate_config);
    else
	logit(log_file, "PANIC unable to open configuration file: %s\n", DefaultConfigFile);
    bridge_mode = 1;
    return;
  }

  buf = (char *)malloc(sizeof(char)*LOW_BUFF*2);
  if (buf == NULL) {
	logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
	bridge_mode = 1;
	fclose(fp);
	return;
  }
  while ((bridge_mode == 0) && (fgets(buf, LOW_BUFF, fp) != NULL)) {
      /* chop newline */
      chomp(buf);
      /* add to regex patterns array */
      add_pattern(buf);
     if (bridge_mode == 1) {
	xfree(buf);
	fclose(fp);
	return;
     }
  }
  xfree(buf);
  if (redirect_url == NULL) {
    logit(log_file, "PANIC No redirection URL set, going to BRIDGE mode\n");
    bridge_mode = 1;
  }
   if (squidguard != NULL) {
    logit(log_file, "LOG Chaining with %s\n", squidguard);
  }
   if (useragent != NULL) {
    logit(log_file, "LOG Anonymizing User-Agent: %s\n", useragent);
  }
  
  if (fclose(fp) != 0)
	logit(log_file, "ERROR Can't close configuration file\n");

  /* Set default values */

  if (clamd_ip == NULL) {
	clamd_ip = (char *) malloc (sizeof (char) * SMALL_CHAR);
	if(clamd_ip == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	xstrncpy(clamd_ip, CLAMD_SERVER, SMALL_CHAR);
  }

  if (clamd_port == NULL) {
	clamd_port = (char *) malloc (sizeof (char) * LOW_CHAR);
	if(clamd_port == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	xstrncpy(clamd_port, CLAMD_PORT, LOW_CHAR);
  }

  if (squid_ip == NULL) {
	squid_ip = (char *) malloc (sizeof (char) * LOW_CHAR);
	if(squid_ip == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	xstrncpy(squid_ip, PROXY_SERVER, LOW_CHAR);
  }

  if (squid_port == NULL) {
	squid_port = (char *) malloc (sizeof (char) * LOW_CHAR);
	if(squid_port == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	xstrncpy(squid_port, PROXY_PORT, LOW_CHAR);
  }

  if (proxy == NULL) {
	proxy = (char *) malloc (sizeof (char) * LOW_BUFF);
	if(proxy == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	snprintf(proxy, LOW_BUFF, "http://%s:%s", squid_ip, squid_port);
  }

}

/* Add regexp expression to patterns array */
void
add_pattern(char *s)
{
	char *first = NULL;
	char *type  = NULL;
	int stored = 0;
	int regex_flags = REG_NOSUB;
	SCPattern currItem;

	/* skip empty and commented lines */
	if ( (xstrnlen(s, LOW_BUFF) == 0) || (strncmp(s, "#", 1) == 0)) return;

	/* Config file directives are construct as follow: name value */  
	type = (char *)malloc(sizeof(char)*LOW_CHAR);
	first = (char *)malloc(sizeof(char)*LOW_BUFF);
	stored = sscanf(s, "%31s %255[^#]", type, first);
  
	if (stored < 2) {
		logit(log_file, "PANIC Bad configuration line for [%s]\n", s);
		bridge_mode = 1;
		xfree(type);
		xfree(first);
		return;
	}
	/* remove extra space or tabulation */
	trim(first);

	/* URl to redirect Squid on virus found */  
	if(strcmp(type, "redirect") == 0) {
		redirect_url = (char *) malloc (sizeof (char) * LOW_BUFF);
		if(redirect_url == NULL) {
			logit(log_file, "PANIC unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(redirect_url, first, LOW_BUFF);
		}
		xfree(type);
		xfree(first);
		return;
	}

	/* Path to chained other Squid redirector, mostly SquidGuard */
	if(strcmp(type, "squidguard") == 0) {
		squidguard = (char *) malloc (sizeof (char) * LOW_BUFF);
		if(squidguard == NULL) {
			logit(log_file, "PANIC unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			if (isPathExists(first) == 0) {
				xstrncpy(squidguard, first, LOW_BUFF);
			} else {
				logit(log_file, "Wrong path to SquidGuard, disabling.\n");
				squidguard = NULL;
			}
		}
		xfree(type);
		xfree(first);
		return;
	}
  
	if(strcmp(type, "debug") == 0) {
		if (debug == 0)
		   debug = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "logredir") == 0) {
		if (logredir == 0)
		   logredir = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "timeout") == 0) {
		timeout = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "stat") == 0) {
		statit = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "maxredir") == 0) {
		maxredir = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	/* Path to log file */
	if(strcmp(type, "logfile") == 0) {
		log_file = (char *) malloc (sizeof (char) * LOW_BUFF);
		if (log_file == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(log_file, first, LOW_BUFF);
		}
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "proxy") == 0) {
		proxy = (char *) malloc (sizeof (char) * LOW_BUFF);
		if(proxy == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(proxy, first, LOW_BUFF);
		}
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "clamd_ip") == 0) {
		clamd_ip = (char *) malloc (sizeof (char) * SMALL_CHAR);
		if (clamd_ip == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(clamd_ip, first, SMALL_CHAR);
		}
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "clamd_port") == 0) {
		clamd_port = (char *) malloc (sizeof (char) * LOW_CHAR);
		if(clamd_port == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(clamd_port, first, LOW_CHAR);
		}
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "clamd_local") == 0) {
		clamd_local = (char *) malloc (sizeof (char) * LOW_BUFF);
		if(clamd_local == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(clamd_local, first, LOW_BUFF);
		}
		xfree(type);
		xfree(first);
		return;
	}

	/* User Agent */
	if(strcmp(type, "useragent") == 0) {
		useragent = (char *) malloc (sizeof (char) * LOW_BUFF);
		if (useragent == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(useragent, first, LOW_BUFF);
		}
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "trust_cache") == 0) {
		trust_cache = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "squid_ip") == 0) {
		squid_ip = (char *) malloc (sizeof (char) * LOW_CHAR);
		if (squid_ip == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(squid_ip, first, LOW_CHAR);
		}
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "squid_port") == 0) {
		squid_port = (char *) malloc (sizeof (char) * LOW_CHAR);
		if(squid_port == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
		} else {
			xstrncpy(squid_port, first, LOW_CHAR);
		}
		xfree(type);
		xfree(first);
		return;
	}
 
	if (strcmp(type, "maxsize") == 0) {
		maxsize = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	/* force case insensitive pattern matching */
	/* so aborti, contenti, regexi are now obsolete */
	regex_flags |= REG_ICASE;
	/* Add extended regex search */
	regex_flags |= REG_EXTENDED;
	/* Fill the pattern type */
	if (strcmp(type, "abort") == 0) {
		currItem.type = ABORT;
	} else if (strcmp(type, "abortcontent") == 0) {
		currItem.type = ABORTCONTENT;
		nohead = 0;
	} else if(strcmp(type, "whitelist") == 0) {
		currItem.type = WHITELIST;
	} else if(strcmp(type, "trustuser") == 0) {
		currItem.type = TRUSTUSER;
	} else if(strcmp(type, "trustclient") == 0) {
		currItem.type = TRUSTCLIENT;
	} else {
		fprintf(stderr, "WARNING: Bad configuration keyword: %s\n", s);
		xfree(type);
		xfree(first);
		return;
	}

	/* Fill the pattern flag */
	currItem.flag = regex_flags;

	/* Fill pattern array */
	currItem.pattern = malloc(sizeof(char)*(strlen(first)+1));
	if (currItem.pattern == NULL) {
		fprintf(stderr, "unable to allocate new pattern in add_to_patterns()\n");
		bridge_mode = 1;
		xfree(type);
		xfree(first);
		return;
	}
	strncpy(currItem.pattern, first, strlen(first) + 1);
	if ((stored = regcomp(&currItem.regexv, currItem.pattern, currItem.flag)) != 0) {
		logit(log_file, "ERROR Invalid regex pattern: %s\n", currItem.pattern);
	} else {
		if (growPatternArray(currItem) < 0) {
			fprintf(stderr, "unable to allocate new pattern in add_to_patterns()\n");
			bridge_mode = 1;
		}
	}
	xfree(type);
	xfree(first);
}

int
growPatternArray (SCPattern item)
{
	void *_tmp = NULL;
        if (pattc == current_pattern_size) {
                if (current_pattern_size == 0)
                        current_pattern_size = PATTERN_ARR_SIZE;
                else
                        current_pattern_size += PATTERN_ARR_SIZE;

		_tmp = realloc(patterns, (current_pattern_size * sizeof(SCPattern)));
                if (!_tmp) {
                        return(-1);
                }

                patterns = (SCPattern*)_tmp;
        }
        patterns[pattc] = item;
        pattc++;

	return(pattc);
}

